/*

GooBer Portable gameboy emulator
Copyright (C) 2002 Ian Smith

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _MEM_H
#define _MEM_H

#include <stdio.h>
#include "misc.h"

typedef unsigned char (*MEMREAD)(unsigned short);
typedef void (*MEMWRITE)(unsigned short, unsigned char);


void memory_init();
void memory_reset();
void memory_cleanup();

/* Encapsulate all memory accesses in these two functions */
unsigned char mem_read(unsigned short a);
void mem_write(unsigned short a, unsigned char b);

void set_rom0_source(unsigned char *p);
void set_rom0_handlers(MEMREAD r, MEMWRITE w);

void set_rom1_source(unsigned char *p);
void set_rom1_handlers(MEMREAD r, MEMWRITE w);

void set_vram_source(unsigned char *p);
void set_vram_handlers(MEMREAD r, MEMWRITE w);

void set_oam_source(unsigned char *p);
void set_oam_handlers(MEMREAD r, MEMWRITE w);

void set_exram_source(unsigned char *p);
void set_exram_handlers(MEMREAD r, MEMWRITE w);

void set_wram1_source(unsigned char *p);
void set_wram1_handlers(MEMREAD r, MEMWRITE w);

void set_wram0_source(unsigned char *p);
void set_hram_source(unsigned char *p);

/* IO is special, it needs no source but it need handlers */
void set_io_handlers(MEMREAD r, MEMWRITE w);


#endif














