/*

Goober Portable Gameboy Emulator
Copyright (C) 2002 Ian Smith

This Program Is Free Software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "gbcpu.h"
#include "intr.h"
#include "lcdc.h"

static gbcpu_context *cpu;
static unsigned char intr_flag=0, intr_enabled=0;

#define INTR_ENABLED(MASK) ((intr_flag & MASK) == (intr_enabled & MASK))

void intr_set_cpu(gbcpu_context *gbcpu)
{
  cpu = gbcpu;
}

/* IO Access functions */
unsigned char intr_read_reg(unsigned char reg)
{
  if (reg == 0xFF) {
    return intr_enabled;
  }
  else if (reg == 0x0F) {
    return intr_flag;
  }
}

void intr_write_reg(unsigned char reg, unsigned char value)
{
  if (reg == 0xFF) {
    intr_enabled = value;
  }
  else if (reg == 0x0F) {
    intr_flag = value;
    intr_request(intr_flag);
  }
}

void intr_request(unsigned char mask)
{
  /* Check for VBLANK interrupt */
  if ( ((mask & INTR_VBLANK)==INTR_VBLANK) && ((intr_enabled & INTR_VBLANK)==INTR_VBLANK) ) {
    if (gbcpu_int(cpu, 0x0040)) intr_flag ^= INTR_VBLANK;
  }

  /* Check for LCDSTAT interrupt */
  if (((mask & INTR_LCDSTAT)==INTR_LCDSTAT) && ((intr_enabled & INTR_LCDSTAT)==INTR_LCDSTAT)) {
    if (gbcpu_int(cpu, 0x0048)) intr_flag ^= INTR_LCDSTAT;
  }

  /* Check for TIMER interrupt */
  if (((mask & INTR_TIMER)==INTR_TIMER) && ((intr_enabled & INTR_TIMER)==INTR_TIMER)) {
    if (gbcpu_int(cpu, 0x0050)) intr_flag ^= INTR_TIMER;
  }

  /* Check for SERIAL interrupt */
  if (((mask & INTR_SERIAL)==INTR_SERIAL) && ((intr_enabled & INTR_SERIAL)==INTR_SERIAL)) {
    if (gbcpu_int(cpu, 0x0058)) intr_flag ^= INTR_SERIAL;
  }

  /* Check for JOYPAD interrupt */
  if (((mask & INTR_JOYPAD)==INTR_JOYPAD) && ((intr_enabled & INTR_JOYPAD)==INTR_JOYPAD)) {
    if (gbcpu_int(cpu, 0x0060)) intr_flag ^= INTR_JOYPAD;
  }
}

int intr_pending(unsigned char i)
{
  if ((intr_flag & i)==i) 
    return 1;
  else
    return 0;
}



